// change zeroClient to whatever object the Project Zero client side code lives in
(function() {
	if(typeof this["zeroClient"] == "undefined") {
		this.zeroClient = {};
	}
})();


zeroClient.CSRFBasic = new (function() {
	// private properties
	var _csrfHttpsCookieName =  "pzerocsrfprotectsec"; // the secure cookie for CSRF protection
	var _csrfHttpCookieName = "pzerocsrfprotect"; // the insecure cookie for CSRF protection
	var _csrfParamName = "Pzcsrf"; // The parameter or form field name for embedding the CSRF secret
	var _csrfHeaderName = "Pzcsrf"; // The header name for embedding the CSRF secret
	var _myHost =  null; // The host from which this document was retrieved
	// TODO: the context root is not yet taken into account!
	var _myPort = -1; 
	var _myScheme = null; // The URL scheme. For CSRF protection we currently only support http, https
	var _amSecure = false; // true if _myScheme is true, else false
	var _httpPort = 80;
	var _httpsPort = 443;
	var _isIE = false;
	var _isSafari = false;
	var that = this;

	var _regex = new RegExp("^((http|https):)?(//([^/?#:]*))?(:([0-9]*))?([^?#]*)(\\?([^#]*))?(#.*)?");

	// initializes all the local variables
	function init() {
		var navversion = navigator.appVersion;
		if (navversion.indexOf("MSIE") >= 0)
			_isIE = true;
		if (navversion.indexOf("Safari") >= 0)
			_isSafari = true;
		// initialize _myHost and _myScheme
		var myurl = window.location.href;
		var myurlparsed = new _ParsedUrl(myurl);
		_myScheme = (myurlparsed.scheme).toLowerCase();
		if (_myScheme == "https") {
			_amSecure = true;
		}
		else {
			_amSecure = false;
		}
		_myHost = (myurlparsed.host).toLowerCase();
		if (myurlparsed.port == null) {
			if (_amSecure) 
				_myPort = _httpsPort;
			else if (_myScheme == "http") {
				_myPort = _httpPort;
			}
		}
		else {
			_myPort = myurlparsed.port;
		}
	}
	init(); // call init()
	
	/*
	For overriding the following constants: csrfHttpsCookieName, csrfHttpCookieName, csrfParamName, csrfHeaderName
		httpPort, httpsPort
	*/
	this.configure = function(/*Object*/ config) {
		if (config.csrfHttpsCookieName != null) {
			_csrfHttpsCookieName = config.csrfHttpsCookieName;
		}
		if (config.csrfHttpCookieName != null) {
			_csrfHttpCookieName = config.csrfHttpCookieName;
		}
		if (config.csrfParamName != null) {
			_csrfParamName = config.csrfParamName;
		}
		if (config.csrfHeaderName != null) {
			_csrfHeaderName = config.csrfHeaderName;
		}
		if (config.httpPort != null) {
			_httpPort = config.httpPort;
		}
		if (config.httpsPort != null) {
			_httpsPort = config.httpsPort;
		}
		// rerun init()
		init();
	}
	//-----------------------------------------------------------------------------
	// getCSRFParamName, getCSRFHeaderName, getCSRFTokenValue are the raw methods for someone not
	// relying on the more complex methods that require less programmer effort.
	/*
	Get the name of the URL parameter or form field that the server expects the CSRF token in
	*/
	this.getCSRFParamName = function() {
		return _csrfParamName;
	}
	/*
	Get the name of the header that the server expects the CSRF token in
	*/
	this.getCSRFHeaderName = function() {
		return _csrfHeaderName;
	}

	/*
	Get the value of the CSRF token for the given URL. May be null if there is no CSRF token
	for that URL, or if the user is not authenticated 
	*/
	this.getCSRFTokenValue = function(/*String*/ url) {
		var urlparsed = new _ParsedUrl(url);
		var whichToken = _analyzeUrl(urlparsed);
		// alert(whichToken);
		if (whichToken == _NONE) {
			return null;
		}
		else {
			return _getToken(whichToken);
		}
	}
	//-----------------------------------------------------------------------------
	// Next level of support for CSRF protection is providede by addCSRFTokenToURL, addCSRFTokenToHeader, 
	// addCSRFTokenToForm, removeCSRFTokenFromForm
	/*
	Add a CSRF token to the given URL. The return value contains the potentially modified URL.
	No token may be added if the user is not authenticated, or there is no CSRF token for that URL
	*/
	this.addCSRFTokenToURL = function(/*String*/ url) {
		var urlparsed = new _ParsedUrl(url);
		var whichToken = _analyzeUrl(urlparsed);
		if (whichToken == _NONE) {
			return url;
		}
		else {
			return _attachTokenToURL(url, urlparsed, whichToken);
		}
	}
	/*
	Add a CSRF token as a header to the given XHR.
	No token may be added if the user is not authenticated, or there is no CSRF token for the URL
	*/
	this.addCSRFTokenToHeader = function(/*String*/ url, /*XHR*/ xhr) {
		var urlparsed = new _ParsedUrl(url);
		var whichToken = _analyzeUrl(urlparsed);
		if (whichToken == _NONE) {
			// alert("addCSRFTokenToHeader:whichToken == _NONE");
			return url;
		}
		else {
			return _attachTokenToHeader(xhr, whichToken);
		}
	}
	/*
	Add a CSRF token as a hidden field to the given form. The form action attribute must be initialized
	to the URL that will be used when the form is submitted.
	No token may be added if the user is not authenticated, or there is no CSRF token for the 
	form action URL.
	*/
	this.addCSRFTokenToForm = function(/*Form*/ form) {
		//IE requires going through getAttributeNode instead of just getAttribute in some form cases, 
		//so use it for all.  See #2844
		var actnNode = form.getAttributeNode("action");
		if (!actnNode)
			return;
		var url = actnNode.value;
		if (!url)
			return;
		var urlparsed = new _ParsedUrl(url);
		var whichToken = _analyzeUrl(urlparsed);
		if (whichToken == _NONE) {
			return;
		}
		var credValue = _getToken(whichToken);
		if (credValue == null)
			return;
		var credName = _csrfParamName;
		if(!form[credName]) {
			var tn;
			if (_isIE) {
				tn = window.document.createElement("<input type='hidden' name='" + credName + "' value='" + credValue + "'>");
				form.appendChild(tn);
			}
			else {
				tn = window.document.createElement("input");
				form.appendChild(tn);
				tn.type = "hidden";
				tn.name = credName;
				tn.value = credValue;
			}
		}
		else {
			form[credName].value = credValue;
		}
	}
	/*
	Remove the hidden field containing the CSRF token. Only need to do this if form is being reused with a different action attribute
	*/
	this.removeCSRFTokenFromForm = function(/*Form*/ form) {
		var key =_csrfParamName;
		var theChildNode;
		if (_isSafari) {
			// cannot use form[key] syntax to find dynamically created nodes
			for(var j = 0; j < form.childNodes.length; j++){
				var temp = form.childNodes[j];
				if (temp.name == key) {
					theChildNode = temp;
					break;
				}
			}
			if (theChildNode != null) 
				theChildNode.parentNode.removeChild(theChildNode);				
		}
		else {
			theChildNode = form[key];
			if (theChildNode != null) {
				theChildNode.parentNode.removeChild(theChildNode);				
				form[key] = null;
			}
		}

	}
	//-----------------------------------------------------------------------------
	// Private helper functions
	var _NONE = 0;
	var _INSECURE = 1;
	var _SECURE = 2;

	// return _NONE if don't attach any token, _INSECURE if attach insecure token and _SECURE if attach secure token
	function _analyzeUrl(/*_ParsedUrl*/ urlparsed) {
		var urlscheme = null;
		var urlIsSecure = false;
		var absoluteurl = false;
		var sameScheme = false;
		if (urlparsed.scheme != null) {
			urlscheme = urlparsed.scheme.toLowerCase();
			if (urlscheme == "https") {
				urlIsSecure = true;
			}
			else if (urlscheme != "http") {
				return _NONE;
			}
			absoluteurl = true;
			sameScheme = (urlscheme == _myScheme);		
		}
		else {
			urlscheme = _myScheme;
			sameScheme = true;
		}
		// alert("absoluteurl=" + absoluteurl + ",sameScheme=" + sameScheme);
		if (!absoluteurl) { // need to see if host is specified or not
			if (urlparsed.host == null) {
				// alert("urlparsed.host==null");
				// it is a relative url with no scheme or host
				if ( (urlparsed.path === null) && (urlparsed.query === null) && (urlparsed.fragment !== null ) ) {
					// it's just a fragment, i.e., it will be locally handled ...
					return _NONE;
				}
				if (_amSecure) {
					return _SECURE;
				}
				else {
					return _INSECURE;
				}
			}
			// alert("urlparsed.host=" + urlparsed.host);
		}
		// have scheme in urlscheme, and urlparsed.host != null. 
		var credChoice = _INSECURE; // which token should be used
		if ((_amSecure) && (sameScheme)) {
			credChoice = _SECURE;
		}
		// now need to see if host matches. unfortunately can only do a partial check
		// for matching ports, only for the case of sameScheme==true. If sameScheme==false,
		// ports are not expected to match and we have no idea if we are going to a different
		// application on the same host, or the same application. 
		if (_myHost == urlparsed.host) {
			if (sameScheme) {
				var port = urlparsed.port;
				if (port == null) {
					// alert("urlparsed.port==null");
					if (_amSecure) { // and since scheme is the same
						port = _httpsPort;
					}
					else // http
						port = _httpPort;
				}
				else {
					// alert("urlparsed.port=" + port);
				}
				if (_myPort != port) { // should have the same ports since scheme was the same
					return _NONE;
				}
			}
			return credChoice;
		}
		return _NONE;
	}

	function _attachTokenToURL(/*String*/ url, /*_ParsedUrl*/ urlparsed, /*Number*/ whichToken) {
		var cred = _getToken(whichToken);
		if (cred != null) {
			if (urlparsed.fragment == null) {
				var newurl = url + ((urlparsed.query == null) ? "?" : "&") + _csrfParamName + "=" + cred;
				return newurl;
			} else {
				var newurl =
				 ((urlparsed.scheme == null) ? "" : (urlparsed.scheme + ":") ) +
				 ((urlparsed.host == null) ? "" :  ("//" + urlparsed.host)  ) +
				 ((urlparsed.port == null) ? "" :  (":" + urlparsed.port)  ) +
				 ((urlparsed.path == null) ? "" :  urlparsed.path  ) +
				 ((urlparsed.query == null) ? "" : urlparsed.query) + 
				 ((urlparsed.query == null) ? "?" : "&") + _csrfParamName + "=" + cred +
				 urlparsed.fragment; 
				return newurl;
			}
			
		}
		else {
			return url;
		}
	}

	function _attachTokenToHeader(/*XHR*/ xhr, /*Number*/ whichToken) {
		var cred = _getToken(whichToken);
		// alert("_attachTokenToHeader:cred=" + cred);
		if (cred != null) {
			xhr.setRequestHeader(_csrfHeaderName, cred);
			// alert("after xhr.setRequestHeader");
		}
	}

	function _getToken(/*Number*/ whichToken) {
		var cookieName = null;
		if (whichToken == _INSECURE) {
			cookieName = _csrfHttpCookieName;
		}
		else if (whichToken == _SECURE) {
			cookieName = _csrfHttpsCookieName;
		}
		else {
			return null;
		}
		var allCookies = window.document.cookie;
		var index = allCookies.lastIndexOf(cookieName + '=');
		if (index == -1) { return null; }
		var startingIndex = index + cookieName.length + 1;
		var endingIndex = allCookies.indexOf(';', startingIndex);
		if (endingIndex == -1) { endingIndex = allCookies.length; }
		var credValue = allCookies.substring(startingIndex, endingIndex);
		// alert(credValue);
		return credValue; 
	}



	function _ParsedUrl(url) {
		var matchedurl = url.match(_regex);
		this.scheme = (matchedurl[2] == "") ? null : matchedurl[2];
		this.host = (matchedurl[4] == "") ? null : matchedurl[4];
		this.port = (matchedurl[6] == "") ? null : matchedurl[6];
		this.path = (matchedurl[7] == "") ? null : matchedurl[7];
		this.query = (matchedurl[8] == "") ? null : matchedurl[8];
		this.fragment = (matchedurl[10] == "") ? null : matchedurl[10];
		// alert("scheme=" + this.scheme + " host=" + this.host + "\n path=" + this.path + " query=" + this.query + "\n port=" + this.port);
	}

})();

